--------------------------------------------------------------------------------
-- Minimal class system

local Object
Object = setmetatable({
  type = "Object",

  init = function(self) end,

  mixin = function(klass, ...)
    for i = 1, select('#', ...) do
      for k, v in pairs((select(i, ...))) do
        if type(klass[k])=="nil" then klass[k]=v end
      end
    end
    return klass
  end,

  subclass = function(klass, child)
    child = child or {}
    child.__index = child
    child.super = klass
    Object.mixin(child, klass)
    return setmetatable(child, getmetatable(klass))
  end,

  new = function(klass, ...)
    local inst = setmetatable({}, klass.__index)
    inst:init(...)
    return inst
  end,
}, {
  __call = function(klass, ...) return klass:new(...) end
})

return setmetatable({
  Object = Object
},{
  __call = function(_, klass)
    return Object:subclass(klass)
  end
})

