--------------------------------------------------------------------------------
-- Input Control Schemes
local Input = {
  clock = 0,
  time = {},
  scheme = {
    left = {"left", "a", "j", "joy_left"},
    right = {"right", "d", "l", "joy_right"},
    start = {"return", "enter", "joy_3", "joy_10"},
    jump = {"z", "o", "joy_1"},
    shoot = {"x", "p", "joy_2"},
    pause = {"escape", "pause", "p", "joy_3", "joy_10"},
  },
  default_joystick = 1,
}

Input.tap = setmetatable({}, {
  __index = function(self, key)
    return Input.time[key] == Input.clock
  end
})

Input.hold = setmetatable({}, {
  __index = function(self, key)
    return Input.time[key] ~= nil
  end
})

Input.len = setmetatable({}, {
  __index = function(self, key)
    local stamp = Input.time[key]
    if stamp then
      return Input.clock - stamp
    else
      return 0
    end
  end
})

function Input:reset()
  -- love.keyboard.setKeyRepeat(0.500, 0.08)
  for k, _ in pairs(self.time) do self.time[k] = nil end
  self.kp = nil
  self.last_key = nil
  self.jb = nil
  self.last_joy = nil
  self.clock = 0
  return self
end

function Input:update(dt)
  self.clock = self.clock + dt
  self.last_key = self.kp
  self.lasy_joy = self.jb
  self.kp = nil
  self.jb = nil
  self:updateScheme()
end

function Input:isDown(key)
  local joy = key:match("joy_(%w+)")
  if joy then
    local joyb = tonumber(joy)
    if joyb then
      return love.joystick.isDown(self.default_joystick, joyb)
    elseif joy == "left" then
      return love.joystick.getAxis(self.default_joystick, 1) <= -1
    elseif joy == "right" then
      return love.joystick.getAxis(self.default_joystick, 1) >= 1
    elseif joy == "up" then
      return love.joystick.getAxis(self.default_joystick, 2) <= -1
    elseif joy == "down" then
      return love.joystick.getAxis(self.default_joystick, 2) >= 1
    end
    return false
  else
    return love.keyboard.isDown(key)
  end
end

function Input:updateScheme()
  for id, tbl in pairs(self.scheme) do
    local found = false
    for _, key in ipairs(tbl) do
      if self:isDown(key) then
        found = true
        self.time[id] = self.time[id] or self.clock
      end
    end
    if not found then
      self.time[id] = nil
    end
  end
end

function Input:keypressed(k, u)
  self.kp = k
end

return Input

