-- LBJ Picks Up His Pants
-- A Game by Inny
-- Todo: Pick a license. 2-clause BSD, MIT, or ZLIB is fine.

--------------------------------------------------------------------------------
-- Acceptable Global Environment pollution
do
  assert(love._version=="0.8.0")
  __TESTING = true
  function NULLFUNC() return end
  log = (__TESTING == true) and print or NULLFUNC
end

--------------------------------------------------------------------------------
-- Libraries
class = require('class')
util = require('util')
Graphics = require('graphics')
Input = require('input')
Animator = require('anim')
Map = require('map')
entity = require('entity')

--------------------------------------------------------------------------------
-- The actual game
do
  Gameplay = class {
    type = "Gameplay",
    scroll_x = 0,
    debug_dt = 0, debug_fps = 0, debug_mem = 0,
  }

  function Gameplay:init()
    self.map = Map("map/level_one.lua")
    self.player = entity.Player(self, 20, 16)
    self.money = entity.Money(self, 128, 16)
    self.enemy = entity.Enemy(self, 64, 16)
    self.entities = { self.player, self.money, self.enemy }
    self.add_entity = {}
    self.remove_entity = {}
    self.left_scroll_bound = 0
    self.right_scroll_bound = self.map.width*16 - Graphics.game_width
  end

  function Gameplay:update(dt)
    for i = 1, #self.entities do
      local ent = self.entities[i]
      for j = i+1, #self.entities do
        local other = self.entities[j]
        if ent:overlaps(other) then
          ent:handleCollision(other)
          other:handleCollision(ent)
        end
      end
      ent:update(dt)
      if ent.y > self.map.height*16 then
        self.remove_entity[ent] = true
      end
    end
      
    for k, _ in pairs(self.add_entity) do
      table.insert(self.entities, k)
      self.add_entity[k] = nil
    end
    local player_died = false
    local i = 1
    while i <= #self.entities do
      if self.remove_entity[self.entities[i]] then
        local ent = table.remove(self.entities, i)
        if ent == self.player then
          player_died = true
        end
      else
        i = i + 1
      end
    end
    for k, _ in pairs(self.remove_entity) do
      self.remove_entity[k] = nil
    end

    if player_died then
      Main:changeState(Gameover())
    end

    self.scroll_x = util.bound(self.player.x - Graphics.game_width * 0.5,
      self.left_scroll_bound, self.right_scroll_bound)
  end

  function Gameplay:draw(dt)
    self.map:draw(self.scroll_x, 0, self.scroll_x/16, 0, 15, 10)
    for i, entity in ipairs(self.entities) do
      entity:draw(dt, self.scroll_x, 0)
    end
    if Main.debug_window_enabled then
      self.debug_dt = self.debug_dt + dt
      if self.debug_dt >= 1 then
        self.debug_dt = self.debug_dt - 1
        self.debug_fps = love.timer.getFPS()
        self.debug_mem = collectgarbage("count")
      end
      Graphics:write(0, 0, "FPS:%i MEM:%ik\nX:%.2f Y:%.2f G:%.1s",
        self.debug_fps, self.debug_mem, self.player.x, self.player.y, tostring(self.player.grounded))
    end
  end

  function Gameplay:addEntity(e)
    self.add_entity[e] = true
  end

  function Gameplay:removeEntity(e)
    self.remove_entity[e] = true
  end

  function Gameplay:resolveCollisions(ent, tx, ty, dx, dy)
    local sx, sy = ent.x, ent.y
    local fdx, fdy, horiz, vert = self.map:resolveEntityCollisions(tx-sx, ty-sy, sx, sy, ent.w, ent.h)
    if (sx+fdx) ~= tx then dx = 0 end
    if (sy+fdy) ~= ty then dy = (dy < 0) and (dy * 0.25) or 0 end
    return sx+fdx, sy+fdy, dx, dy, (vert=="bottom")
  end

end

--------------------------------------------------------------------------------
-- Where the Logo and stuff is shown
do
  TitleScreen = class {
    type = "TitleScreen",
    main_text = util.reflow("LBJ PICKS UP HIS PANTS", 6),
    press_text = util.reflow("PRESS ENTER", 6),
    timer = 0,
  }

  function TitleScreen:init()
    --
  end

  function TitleScreen:update(dt)
    if Input.tap.start then
      Main:changeState(Gameplay())
    end
  end

  function TitleScreen:draw(dt)
    self.timer = (self.timer + dt)

    local main_len = math.min((self.timer)*12, self.main_text:len())
    Graphics:drawImage("lbjtitle", 0, 0)
      :setColor(Graphics.Color.Black)
      :write(4, 80, self.main_text:sub(1, main_len))

    if self.timer > 2 then
      local press_len = math.min((self.timer-2)*12, self.press_text:len())
      Graphics:setColor(Graphics.Color.White)
        :write(198, 140, self.press_text:sub(1, press_len))
    end
  end
end

--------------------------------------------------------------------------------
-- When we've had our fun
do
  Gameover = class {
    type = "Gameover",
    timer = 0,
  }

  function Gameover:init()
    --
  end

  function Gameover:update(dt)
    if Input.tap.start then
      Main:changeState(TitleScreen())
    end
  end

  function Gameover:draw(dt)
    self.timer = self.timer + dt
    local len = 1 + math.min(self.timer*4, 8)
    Graphics:drawImage("lbjinverted", 0, 0)
      :setColor(Graphics.Color.Black)
      :write(172, 148, ("GAMEOVER"):sub(1, len))
  end
end

--------------------------------------------------------------------------------
-- The primary state machine
do
  Main = {
    dt = 0,
    debug_window_enabled = true,

    icon_bounds = {
      money   = { x =  0, y =  0, w = 16, h = 16, u =  7, v = 15 },
      knife_1 = { x =  0, y = 16, w =  8, h =  8, u =  3, v = 19 },
      knife_2 = { x =  8, y = 16, w =  8, h =  8, u = 11, v = 19 },
      knife_3 = { x = 16, y = 16, w =  8, h =  8, u = 19, v = 19 },
      knife_4 = { x = 24, y = 16, w =  8, h =  8, u = 27, v = 19 },

    },
  }

  function Main:init(args)
    local first = _G[args[2]] or TitleScreen
    self.state = first()

    entity.init()
    Graphics:loadSpriteBank("icons", "gfx/icons.png", self.icon_bounds)
  end

  function Main:update(dt)
    self.dt = dt
    self.state:update(dt)
  end

  function Main:draw()
    self.state:draw(self.dt)
  end

  function Main:changeState(to)
    log("Changing state to", to.type, to)
    self.state = to
  end
end

--------------------------------------------------------------------------------
-- The event handlers
do
  function love.load(args)
    Graphics:init()
    Input:reset()
    Main:init(args or {})
  end

  function love.update(dt)
    if dt >= 0.5 then dt = 0.5 end
    Input:update(dt)
    Main:update(dt)
  end

  function love.draw()
    Graphics:start()
    Main:draw()
    Graphics:stop()
  end

  function love.keypressed(k, u)
    if k=="f2" then Graphics:saveScreenshot() end
    if k=="f3" then Main.debug_window_enabled = not Main.debug_window_enabled end
    if k=="f5" then Graphics:setNextScale() end
    if k=="f10" then love.event.quit() end
    Input:keypressed(k, u)
  end
end

