--------------------------------------------------------------------------------
-- General utilities
util = {}

function util.rectOverlaps(ax1, ay1, aw, ah, bx1, by1, bw, bh)
  local HINT = 0.001
  local ax2, ay2 = ax1+aw-HINT, ay1+ah-HINT
  local bx2, by2 = bx1+bw-HINT, by1+bh-HINT
  return not ((ax1 > bx2) or (bx1 > ax2) or (ay1 > by2) or (by1 > ay2))
end

function util.reflow(str, limit)
  local t, i, nl, sp = {}, 0, 0, 0
  limit = limit or 20
  for c in str:gmatch(".") do
    i = i + 1
    t[i] = c
    if (c == " ") then sp = i
    elseif (c == "\n") then sp, nl = i, i end
    if (i-nl) > limit then t[sp], nl = "\n", sp end
  end
  return table.concat(t)
end

function util.lerp(y1, y2, x)
  return y1 + (x * (y2 - y1))
end

function util.round(n)
  if n >= 0 then return math.floor(n+0.5) else return math.ceil(n-0.5) end
end

function util.bound(n, l, r)
  if n > r then return r elseif n < l then return l else return n end
end

function util.compare(n, v, precision)
  precision = precision or 0.00001
  return (n >= v-precision) and (n <= v+precision)
end

function util.abs(n)
  if n < 0 then return -n else return n end
end

function util.sign(n)
  if n < 0 then return -1 else return 1 end
end

function util.loadLuaFile(filename)
  local ok, chunk, data
  ok, chunk = pcall(love.filesystem.load, filename)
  if not ok then error("(loadLuaFile load): "..tostring(chunk)) end
  ok, data = pcall(chunk)
  if not ok then error("(loadLuaFile call): "..tostring(data)) end
  return data
end

return util

